@@ -11,7 +11,7 @@ import yaml
 from . import errors
 import pickle
 import shutil
-import multiprocessing
+import concurrent.futures
 import copy
 from typing import Dict, NewType, Mapping, Optional
 from atomicwrites import atomic_write
@@ -68,7 +68,7 @@ RESOURCES = {
         'apps': 'https://raw.githubusercontent.com/tapis-project/tapipy/prod/tapipy/resources/openapi_v3-apps.yml'
     },
     'prod': {
-        'actors': 'https://raw.githubusercontent.com/TACC/abaco/dev-v3/docs/specs/openapi_v3.yml',               
+        'actors': 'https://raw.githubusercontent.com/TACC/abaco/prod-v3/docs/specs/openapi_v3.yml',               
         'authenticator': 'https://raw.githubusercontent.com/tapis-project/authenticator/prod/service/resources/openapi_v3.yml',
         'meta': 'https://raw.githubusercontent.com/tapis-project/tapis-client-java/prod/meta-client/src/main/resources/metav3-openapi.yaml',
         'files': 'https://raw.githubusercontent.com/tapis-project/tapis-files/prod/api/src/main/resources/openapi.yaml',
@@ -81,6 +81,20 @@ RESOURCES = {
         'jobs': 'https://raw.githubusercontent.com/tapis-project/tapis-client-java/prod/jobs-client/src/main/resources/JobsAPI.yaml',
         'apps': 'https://raw.githubusercontent.com/tapis-project/openapi-apps/prod/AppsAPI.yaml'
     },
+    'staging': {
+        'actors': 'https://raw.githubusercontent.com/TACC/abaco/dev-v3/docs/specs/openapi_v3.yml',               
+        'authenticator': 'https://raw.githubusercontent.com/tapis-project/authenticator/staging/service/resources/openapi_v3.yml',
+        'meta': 'https://raw.githubusercontent.com/tapis-project/tapis-client-java/staging/meta-client/src/main/resources/metav3-openapi.yaml',
+        'files': 'https://raw.githubusercontent.com/tapis-project/tapis-files/staging/api/src/main/resources/openapi.yaml',
+        'sk': 'https://raw.githubusercontent.com/tapis-project/tapis-client-java/staging/security-client/src/main/resources/SKAuthorizationAPI.yaml',
+        'streams': 'https://raw.githubusercontent.com/tapis-project/streams-api/staging/service/resources/openapi_v3.yml',
+        'systems': 'https://raw.githubusercontent.com/tapis-project/openapi-systems/staging/SystemsAPI.yaml',
+        'tenants': 'https://raw.githubusercontent.com/tapis-project/tenants-api/staging/service/resources/openapi_v3.yml',
+        'tokens': 'https://raw.githubusercontent.com/tapis-project/tokens-api/staging/service/resources/openapi_v3.yml',
+        'pgrest': 'https://raw.githubusercontent.com/tapis-project/paas/staging/pgrest/resources/openapi_v3.yml',
+        'jobs': 'https://raw.githubusercontent.com/tapis-project/tapis-client-java/staging/jobs-client/src/main/resources/JobsAPI.yaml',
+        'apps': 'https://raw.githubusercontent.com/tapis-project/openapi-apps/staging/AppsAPI.yaml'
+    },
     'dev': {
         'actors': 'https://raw.githubusercontent.com/TACC/abaco/dev-v3/docs/specs/openapi_v3.yml',
         'authenticator': 'https://raw.githubusercontent.com/tapis-project/authenticator/dev/service/resources/openapi_v3.yml',
@@ -91,7 +105,7 @@ RESOURCES = {
         'systems': 'https://raw.githubusercontent.com/tapis-project/openapi-systems/dev/SystemsAPI.yaml',
         'tenants': 'https://raw.githubusercontent.com/tapis-project/tenants-api/dev/service/resources/openapi_v3.yml',
         'tokens': 'https://raw.githubusercontent.com/tapis-project/tokens-api/dev/service/resources/openapi_v3.yml',
-        'pgrest': 'https://raw.githubusercontent.com/tapis-project/paas/prod/pgrest/resources/openapi_v3.yml',
+        'pgrest': 'https://raw.githubusercontent.com/tapis-project/paas/dev/pgrest/resources/openapi_v3.yml',
         'jobs': 'https://raw.githubusercontent.com/tapis-project/tapis-client-java/dev/jobs-client/src/main/resources/JobsAPI.yaml',
         'apps': 'https://raw.githubusercontent.com/tapis-project/openapi-apps/dev/AppsAPI.yaml'
     }
@@ -114,7 +128,9 @@ def _get_specs(resources: Resources, spec_dir: str = None, download_latest_specs
 def download_and_pickle_spec_dicts(resources: Resources, spec_dir: str, download_latest_specs: bool) -> None:
     """
     Function that calls threads to download and pickle specs.
-    Cuts wait time for 9 specs from 4s to 0.7s
+    Not parallel: 4s for 9 specs
+    Threads: Untested
+    Multiprocessing: 0.7s for 9 specs
     """
     # Get list of specs and check which we need to download
     # We download if the file name requested does not exist
@@ -129,12 +145,19 @@ def download_and_pickle_spec_dicts(resources: Resources, spec_dir: str, download
             if not full_spec_name in os.listdir(spec_dir):
                 urls_to_download.append([resource_name, url, spec_path])
     
+    # Set off some parallel threads cause it's quick.
+    # Switched from multiprocessing due to some startup warnings when import Tapipy in scripts
+    if urls_to_download:
+        POOL_SIZE = os.environ.get('POOL_SIZE', 16)
+        with concurrent.futures.ThreadPoolExecutor(max_workers=POOL_SIZE) as executor:
+            executor.map(_thread_download_spec_dict, urls_to_download)
+
     # Set off some parallel processes cause it's quick.
-    POOL_SIZE = os.environ.get('POOL_SIZE', 16)
-    pool = multiprocessing.Pool(processes=POOL_SIZE)
-    pool.map(_thread_download_spec_dict, urls_to_download)
-    pool.close()
-    pool.join()
+    #POOL_SIZE = os.environ.get('POOL_SIZE', 16)
+    #pool = multiprocessing.Pool(processes=POOL_SIZE)
+    #pool.map(_thread_download_spec_dict, urls_to_download)
+    #pool.close()
+    #pool.join()
 
 
 def _thread_download_spec_dict(resource_info: ResourceInfo) -> None:
